/*
 * Copyright (c) 2009 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Brian Hill, July 2009
 * bmp.c: Procedures for display of a Windows Bitmap 
 * 
 */

#include <string.h>
#include <stdio.h>
#include "platform_config.h"

#define DPY_BPP  4 /* Display memory is 4 bytes per pixel */
#define BMP_BPP  3 /* 24-bit BMP images supported here; 3 bytes per pixel */

extern unsigned frameBufferBaseAddr;

/*
 * Return the byte-swapped 16-bit input.
 * BMP fields are little-endian
 */
static inline unsigned
swap16 (unsigned short *in)
{
    return ((*in & 0xFF00) >> 8) | ((*in & 0x00FF) << 8);
}

/*
 * Return the byte-swapped 32-bit input.
 * BMP fields are little-endian
 */
static inline unsigned
swap32 (unsigned *in)
{
    unsigned short *lsw, *msw;

    /*
     * Swap 16 bits at a time.  In this way 32 bit fields which are only
     * 16 bit aligned may be handled successfully.
     */
    lsw = (unsigned short*)in;
    msw = (unsigned short*)((char*)in+2);
    return (swap16(lsw) | (swap16(msw) << 16));
}

/* Widows bitmap file field offsets */
#define BM_MAGIC_OFF          0  /* 0 */
#define BM_FILE_SIZE_OFF      2  /* 0+2 */
#define BM_IMAGE_OFFSET_OFF   10 /* 0+2+4+2+2 */
#define BM_DIB_HDR_SZ_OFF     14 /* 0+2+4+2+2+4 */
#define BM_WIDTH_OFF          18 /* 0+2+4+2+2+4+4 */
#define BM_HEIGHT_OFF         22 /* 0+2+4+2+2+4+4+4 */
#define BM_CLRPN_OFF          26 /* 0+2+4+2+2+4+4+4+4 */
#define BM_BPP_OFF            28 /* 0+2+4+2+2+4+4+4+4+2 */
#define BM_COMPRS_MTD_OFF     30 /* 0+2+4+2+2+4+4+4+4+2+2 */
#define BM_IMAGE_SZ_OFF       34 /* 0+2+4+2+2+4+4+4+4+2+2+4 */

/*
 * Fish out the desired windows bitmap header. The header fields are in
 * little-endian order, so swap the return value.
 */
static unsigned
windows_bm_field (const char *hdr, unsigned off, unsigned sz)
{
    if (sz == 2) {
        return (swap16((void*)(hdr + off)));
    } else {
        return (swap32((void*)(hdr + off)));
    }

    return 0;
}

unsigned
windows_bm_width (const char *bm_hdr)
{
    return windows_bm_field(bm_hdr, BM_WIDTH_OFF, 4);
}

unsigned
windows_bm_height (const char *bm_hdr)
{
    return windows_bm_field(bm_hdr, BM_HEIGHT_OFF, 4);
}

/*
 * Print out the Windows Bitmap Header
 */
void
print_bm_hdr (const char *bm_hdr)
{
    xil_printf("Windows Bitmap Header: 0x%08x\n\r", bm_hdr);
    xil_printf("magic:        0x%04x\n\r", *(unsigned short*)bm_hdr);
    xil_printf("file_size:    %d\n\r", 
               windows_bm_field(bm_hdr, BM_FILE_SIZE_OFF, 4));
    xil_printf("image_offset: 0x%04x\n\r",
               windows_bm_field(bm_hdr, BM_IMAGE_OFFSET_OFF, 4));
    xil_printf("dib_hdr_sz:   0x%04x\n\r",
               windows_bm_field(bm_hdr, BM_DIB_HDR_SZ_OFF, 4));
    xil_printf("width:        %d\n\r", 
               windows_bm_field(bm_hdr, BM_WIDTH_OFF, 4));
    xil_printf("height:       %d\n\r",
               windows_bm_field(bm_hdr, BM_HEIGHT_OFF, 4));
    xil_printf("bpp:          %d\n\r",
               windows_bm_field(bm_hdr, BM_BPP_OFF, 2));
    xil_printf("comprss_mtd:  %d\n\r",
               windows_bm_field(bm_hdr, BM_COMPRS_MTD_OFF, 4));
    xil_printf("image_size:   %d\n\r",
               windows_bm_field(bm_hdr, BM_IMAGE_SZ_OFF, 4));
}

/*
 * Display a windows bitmap image. The image size must not exceed the
 * display resolution (no scaling performed).
 */
void
display_wbm (const char *bmp, int dstx, int dsty)
{
    int srcx, srcy;
    int imgwidth, imgheight;
    char *dest;
    const char *bmpimage, *src;
    int srclinewidth;

    imgwidth = windows_bm_width(bmp);
    imgheight = windows_bm_height(bmp);

    if (imgwidth > VIDEO_X_RES) {
        xil_printf("BMP: horizontal resolution %d unsupported.\n\r",
            windows_bm_field(bmp, BM_WIDTH_OFF, 4));
        return;
    }
    if (imgheight > VIDEO_Y_RES) {
        xil_printf("BMP: vertical resolution %d unsupported.\n\r",
            windows_bm_field(bmp, BM_HEIGHT_OFF, 4));
        return;
    }
    if (windows_bm_field(bmp, BM_BPP_OFF, 2) != 24) {
        xil_printf("BMP: %d bpp unsupported; only 24 bpp supported.\n\r",
            windows_bm_field(bmp, BM_BPP_OFF, 2));
        return;
    }

    /* Start of bitmap data in the BMP */
    bmpimage = bmp + windows_bm_field(bmp, BM_IMAGE_OFFSET_OFF, 4);

    srclinewidth = imgwidth * 3;
    /* Round line length up to a multiple of 4 */
    if (srclinewidth % 4) {
        srclinewidth = (srclinewidth + 4) & ~3;
    }

    for (srcy = 0; srcy < imgheight; srcy++) {
        /* Determine the start address of the present row of pixel data.
         * The display is top to bottom, as you would expect.  The BMP is
         * bottom to top, which is retarded.
         */
        dest = (dsty * VIDEO_X_RES * DPY_BPP) +
               (char*)frameBufferBaseAddr + (dstx * DPY_BPP);
        src = bmpimage + ((imgheight - srcy - 1) * srclinewidth);
        for (srcx = 0; srcx < imgwidth; srcx++) {
            dest[0] = src[0]; // B
            dest[1] = src[1]; // G
            dest[2] = src[2]; // R
            dest += DPY_BPP;
            src += BMP_BPP;
        }

        dsty++;
    }
}

/*
 * Scale a single horizontal line of output (Bresenham)
 */
static void
scale_line (char *target, const char *source, int src_width, int tgt_width)
{
    int num_pixels;
    int int_part;
    int fract_part;
    int E;

    num_pixels = tgt_width;
    int_part = src_width / tgt_width;
    fract_part = src_width % tgt_width;
    E = 0;

    while (num_pixels-- > 0) {
        target[0] = source[0]; // B
        target[1] = source[1]; // G
        target[2] = source[2]; // R
        target += DPY_BPP;
        source += (BMP_BPP * int_part);
        E += fract_part;
        if (E >= tgt_width) {
            E -= tgt_width;
            source += BMP_BPP;
        }
    }
}

#define min(a, b) \
           ((a < b) ? a : b)

/*
 * Display a windows bitmap image. 
 * Image is scaled to the display resolution (with original image aspect
 * ration maintained).
 * 
 * Bresenham 2D (bilinear) interpolation.
 */
void
display_wbm_scaled (const char *bmp)
{
    const char *bmpimage;
    const char *source;
    const char *prev_source;
    char *target;
    int   imgwidth, imgheight;
    int   srclinewidth;
    int   num_rows;
    int   int_part;
    int   fract_part;
    int   E;
    int   tgt_width;
    int   tgt_height;
    float aspect;

    prev_source = 0;
    /* Source image dimmensions from BMP header */
    imgwidth = windows_bm_width(bmp);
    imgheight = windows_bm_height(bmp);

    if (windows_bm_field(bmp, BM_BPP_OFF, 2) != 24) {
        xil_printf("BMP: %d bpp unsupported; only 24 bpp supported.\n\r",
            windows_bm_field(bmp, BM_BPP_OFF, 2));
        return;
    }

    srclinewidth = imgwidth * BMP_BPP;
    /* BMP: Round line length up to a multiple of 4 */
    if (srclinewidth % 4) {
        srclinewidth = (srclinewidth + 4) & ~3;
    }

    /* Determine target image size. Maintain image aspect ratio. */
    aspect = (float)imgwidth / (float)imgheight;
    tgt_width  = min(VIDEO_X_RES, (float)VIDEO_Y_RES * aspect);
    tgt_height = min(VIDEO_Y_RES, (float)VIDEO_X_RES / aspect);
    xil_printf("Scaling output image to %dx%d\n\r", tgt_width, tgt_height);

    /* Scaling parameters */
    num_rows = tgt_height;
    int_part = (imgheight / tgt_height) * srclinewidth;
    fract_part = imgheight % tgt_height;
    E = 0;

    /* Start of bitmap data in the BMP */
    bmpimage = bmp + windows_bm_field(bmp, BM_IMAGE_OFFSET_OFF, 4);

    source = bmpimage + ((imgheight - 1) * srclinewidth);
    target = (char*)frameBufferBaseAddr;

    /* Erase the 1st display lines up to start of centered image */
    bzero(target, (VIDEO_X_RES - tgt_width) * DPY_BPP);
    bzero(target, ((VIDEO_Y_RES - tgt_height) / 2) * DPY_BPP * VIDEO_X_RES);
    /* center image on display */
    target += ((VIDEO_X_RES - tgt_width) / 2) * DPY_BPP;
    target += ((VIDEO_Y_RES - tgt_height) / 2) * DPY_BPP * VIDEO_X_RES;

    /* Scale into each output row */
    while (num_rows-- > 0) {
        /* Scale this row */
        if (source == prev_source) {
            memcpy(target, target - (VIDEO_X_RES * DPY_BPP),
                   tgt_width * DPY_BPP);
        } else {
            scale_line(target, source, imgwidth, tgt_width);
            prev_source = source;
        } 

        /* If the image doesn't fill the display, blank the edges */
        bzero(target + (tgt_width * DPY_BPP),
              (VIDEO_X_RES - tgt_width) * DPY_BPP);

        /* Next output row */
        target += (VIDEO_X_RES * DPY_BPP);

        /* Next input row.
         * BMP images are stored bottom-to-top. Retarded.
         */
        source -= int_part;
        E += fract_part;
        if (E >= tgt_height) {
            E -= tgt_height;
            source -= srclinewidth;
        }
    }

    /* Blank any bottom unused portion of the screen */
    bzero(target, ((VIDEO_Y_RES - tgt_height) / 2) * DPY_BPP * VIDEO_X_RES);
}

