// DISCLAIMER OF LIABILITY
//
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a
// license to use this text/file solely for design, simulation,
// implementation and creation of design files limited
// to Xilinx devices or technologies. Use with non-Xilinx
// devices or technologies is expressly prohibited and
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information
// "as-is" solely for use in developing programs and
// solutions for Xilinx devices, with no obligation on the
// part of Xilinx to provide support. By providing this design,
// code, or information as one possible implementation of
// this feature, application or standard, Xilinx is making no
// representation that this implementation is free from any
// claims of infringement. You are responsible for
// obtaining any rights you may require for your implementation.
// Xilinx expressly disclaims any warranty whatsoever with
// respect to the adequacy of the implementation, including
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied
// warranties of merchantability or fitness for a particular
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are
// done at the users sole risk and will be unsupported.
//
//
// Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part
// of this text at all times.
//
//******************************************************************************/
// Filename            : vsk_iic.c
// $Revision:: 4315   $: Revision of last commit
// $Date:: 2008-10-22#$: Date of last commit
// Description         : VSK IIC controller driver file
//******************************************************************************/

#include "vsk_iic.h"

XStatus vsk_iic_core_init(Xuint32 CoreAddress);

/******************************************************************************
* This function initializes the IIC core.
*
* @param    None.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void vsk_iic_init(void)
{
  XStatus status;

  // Initialize the Carrier Board IIC Core
  status = vsk_iic_core_init(VIDEO_OUT_IIC_BASE_ADDRESS);
  if(status != XST_SUCCESS) { print("Error with Video_Out IIC core init\r\n"); return; }

  // Release Chrontel 7301C from Reset
  //	GPO Bit 31 (LSB) is wired to active low reset pin DVI_RESET_B
  XIo_Out32((VIDEO_OUT_IIC_BASE_ADDRESS + 0x124),0xFFFFFFFF);

  // Initialize the FMC-Video board IIC Core
//  status = vsk_iic_core_init(FMC_CARD_IIC_BASE_ADDRESS);
  if(status != XST_SUCCESS) { print("Error with FMC-Video IIC core init\r\n"); return; }
}


/******************************************************************************
* The function uses the low level driver of IIC to initialize the core.
*
* @param    BaseAddress contains the address of the IIC core.
*
* @return   XST_SUCCESS if successful else XST_FAILURE.
*
* @note     None.
*
******************************************************************************/
XStatus vsk_iic_core_init(Xuint32 CoreAddress)
{
   XStatus Status;

   /*
    * Initialize the IIC Core.
    */
   Status = XIic_DynInit(CoreAddress);
   if(Status != XST_SUCCESS)
   {
       return XST_FAILURE;
   }

   return XST_SUCCESS;
}


/******************************************************************************
* This function writes a buffer of bytes to the IIC chip.
*
* @param    CoreAddress contains the address of the IIC core.
* @param    ChipAddress contains the address of the chip.
* @param    RegAddress contains the address of the register to write to.
* @param    BufferPtr contains the address of the data to write.
* @param    ByteCount contains the number of bytes in the buffer to be written.
*           Note that this should not exceed the page size as noted by the
*           constant PAGE_SIZE.
*
* @return   The number of bytes written, a value less than that which was
*           specified as an input indicates an error.
*
* @note     None.
*
******************************************************************************/
Xuint8  vsk_iic_write(Xuint32 CoreAddress, Xuint8 ChipAddress, Xuint8 RegAddress,
                  Xuint8 *BufferPtr, Xuint8 ByteCount)
{
  Xuint8 SentByteCount;
  Xuint8 WriteBuffer[IIC_PAGE_SIZE + 1];
  Xuint8 Index;
  Xuint8 StatusReg;

  // Make sure all the Fifo's are cleared and Bus is Not busy.
  do
  {
    StatusReg = XIic_ReadReg(CoreAddress, XIIC_SR_REG_OFFSET);
    StatusReg = StatusReg & (XIIC_SR_RX_FIFO_EMPTY_MASK |
                             XIIC_SR_TX_FIFO_EMPTY_MASK |
                             XIIC_SR_BUS_BUSY_MASK);
  } while (StatusReg != (XIIC_SR_RX_FIFO_EMPTY_MASK |
			                XIIC_SR_TX_FIFO_EMPTY_MASK));

  /*
   * A temporary write buffer must be used which contains both the address
   * and the data to be written, put the address in first
   */
  WriteBuffer[0] = RegAddress;

  /*
   * Put the data in the write buffer following the address.
   */
  for (Index = 0; Index < ByteCount; Index++)
  {
    WriteBuffer[Index + 1] = BufferPtr[Index];
  }

  /*
   * Write data at the specified address.
   */
  SentByteCount = XIic_DynSend(CoreAddress, ChipAddress, WriteBuffer,
                               ByteCount + 1, XIIC_STOP);
  if (SentByteCount < 1) { SentByteCount = 1; }

  // Return the number of bytes written.
  return SentByteCount - 1;
}


/******************************************************************************
* This function reads a number of bytes from an IIC chip into a
* specified buffer.
*
* @param    CoreAddress contains the address of the IIC core.
* @param    ChipAddress contains the address of the IIC core.
* @param    RegAddress contains the address of the register to write to.
* @param    BufferPtr contains the address of the data buffer to be filled.
* @param    ByteCount contains the number of bytes in the buffer to be read.
*           This value is constrained by the page size of the device such
*           that up to 64K may be read in one call.
*
* @return   The number of bytes read. A value less than the specified input
*           value indicates an error.
*
* @note     None.
*
******************************************************************************/
Xuint8  vsk_iic_read(Xuint32 CoreAddress, Xuint8 ChipAddress, Xuint8 RegAddress,
                 Xuint8 *BufferPtr, Xuint8 ByteCount)
{
  Xuint8 ReceivedByteCount = 0;
  Xuint8 SentByteCount = 0;
  Xuint8 StatusReg;
  XStatus TestStatus=XST_FAILURE;
  Xint32 cnt = 0;

  // Make sure all the Fifo's are cleared and Bus is Not busy.
  do
  {
    StatusReg = XIic_ReadReg(CoreAddress, XIIC_SR_REG_OFFSET);
    StatusReg = StatusReg & (XIIC_SR_RX_FIFO_EMPTY_MASK |
	                          XIIC_SR_TX_FIFO_EMPTY_MASK |
                             XIIC_SR_BUS_BUSY_MASK);
  } while (StatusReg != (XIIC_SR_RX_FIFO_EMPTY_MASK |
			                XIIC_SR_TX_FIFO_EMPTY_MASK));
  // Position the Read pointer to specific location.
  do
  {
    StatusReg = XIic_ReadReg(CoreAddress, XIIC_SR_REG_OFFSET);
    if(!(StatusReg & XIIC_SR_BUS_BUSY_MASK))
    {
      SentByteCount = XIic_DynSend(CoreAddress, ChipAddress,
                                  (Xuint8 *)&RegAddress, 1,
    								        XIIC_REPEATED_START);
    }
    cnt++;
  }while(SentByteCount != 1 && (cnt < 100000));

  // Error writing chip address so return SentByteCount
  if (SentByteCount < 1) { return SentByteCount; }

  // Receive the data.
  ReceivedByteCount = XIic_DynRecv(CoreAddress, ChipAddress, BufferPtr,
                                   ByteCount);

  // Return the number of bytes received.
  return ReceivedByteCount;
}


